/**
 * Project - IMS 13 simulation, stochastic process
 * Milan Zavoral, xzavor02
 * Petr Bednar, xbedna46
 */

#include <cstdio>
#include <iostream>
#include "generators.h"
#include "conversions.h"

#define ArraySize(array) sizeof(array) / sizeof(*(array))

using namespace std;

namespace
{

/* Pointer to generator */
typedef double (*fptr)(double, double, double, double);

/**
 * Program parameters
 */
struct progParams
{
  fptr generator;         ///< generator
  size_t samples;         ///< number of required samples
  FILE* outFile;          ///< output file
  size_t startValue;      ///< start value in histogram
  size_t bucket;          ///< width of one class in histogram
  size_t classes;         ///< number of classes in histogram
  double gen1;            ///< first parameter of generator
  double gen2;            ///< second parameter of generator
  double gen3;            ///< third parameter of generator
  double gen4;            ///< fourth parameter of generator

  progParams() : generator(NULL),
                  samples(0),
                  outFile(NULL),
                  startValue(0),
                  bucket(0),
                  classes(0),
                  gen1(0.0),
                  gen2(0.0),
                  gen3(0.0),
                  gen4(0.0)
  {

  }

  ~progParams()
  {
    if(outFile)
    {
      fclose(outFile);
    }
  }
};

/**
 * Parameter processing
 * @param argc Number of parameters
 * @param argv Vector of parameters
 * @param params Structure for store parameters
 * @return @c true if processing was completed successfully, @c false otherwise
 */
bool doParams(int argc, char** argv, progParams &params)
{
  if(argv == NULL || argc != 7)
  {
    return false;
  }

  const string type = static_cast<string>(argv[1]);
  if(type == "Auta")
  {
    params.generator = auta;
    // best parameters for generator "auta"
    params.gen1 = 23.0;
    params.gen2 = 4.5;
    params.gen3 = 3.6;
    params.gen4 = 0.4;
  }
  else if(type == "Bankomat")
  {
    params.generator = bankomat;
    // best parameters for generator "bankomat"
    params.gen1 = 0.55;
    params.gen2 = 0.62;
    params.gen3 = 0.012;
    params.gen4 = 1.15;
  }
  else if(type == "Prepazka")
  {
    params.generator = prepazka;
    // best parameters for generator "prepazka"
    params.gen1 = 0.17;
    params.gen2 = 0.8;
    params.gen3 = 0.9;
    params.gen4 = 0.93;
  }
  else
  {
    return false;
  }

  params.outFile = fopen(argv[6], "w");
  if(!params.outFile)
  {
    return false;
  }
  string par;
  size_t numParams[] = {0, 0, 0, 0};
  for(size_t i = 0; i < ArraySize(numParams); ++i)
  {
    par = static_cast<string>(argv[i + 2]);
    if(!strToNum(par, numParams[i], dec))
    {
      return false;
    }
  }
  params.samples = numParams[0];
  params.startValue = numParams[1];
  params.bucket = numParams[2];
  params.classes = numParams[3];

  return true;
}

} /* namespace */

/**
 * Main function
 * @param argc Number of parameters
 * @param argv Vector of parameters
 * @return Status of application
 */
int main(int argc, char** argv)
{
  progParams params;
  if(!doParams(argc, argv, params))
  {
    cerr  << "Usage: " << argv[0] << "   GENERATOR   NUMBER_OF_SAMPLES   HISTO_START   HISTO_BUCKET   HISTO_CLASSES   OUTPUT_FILE\n\n"
          << "GENERATOR = {Auta, Bankomat, Prepazka}\n"
          << "NUMBER_OF_SAMPLES - unsigned integer number representing number of samples which will be generated\n"
          << "HISTO_START - unsigned integer number representing start value of histogram\n"
          << "HISTO_BUCKET - unsigned integer number representing width of one class in histogram\n"
          << "HISTO_CLASSES - unsigned integer number representing number of classes in histogram \n"
          << "OUTPUT_FILE - string representing name of output file\n\n"
          << "(c) 2013 P. Bednar, M. Zavoral\n"
          << "Brno University of Technology, Faculty of Information Technology\n";
    return 1;
  }

  double random;
  for(size_t i = 0; i < params.samples; ++i)
  {
    random = params.generator(params.gen1, params.gen2, params.gen3, params.gen4);
    fprintf(params.outFile, "%.1f\n", random);
  }
  return 0;
}
